#!/bin/bash
script_name=`basename $0`
script_dir=`dirname $0`
nupyf=/usr/sbin/nupyf
output_dir=/tmp/nupyf-tests
iptables=/sbin/iptables

function print_usage {
  echo ""
  echo "Usage:"
  echo "  $script_name [-hdc] [<testcase> [<testcase> ...]]"
  echo ""
  echo "Options:"
  echo "  -h : Prints this help."
  echo "  -d : Does not delete output files after failed test at $output_dir."
  echo "  -c : Continues the test procedure even if a test fails."
  echo ""
  echo "Arguments:"
  echo "  <testcase> : a testcase is a subdirectory included in the $script_dir."
  echo ""
  echo "If no argument is given, takes all testcases."
}

function test_file {
  acl=$1
  if [ -e "$output_dir" ]
  then
    if [ -d "$output_dir" ]
    then
      rm -rf "$output_dir"
    else
      echo "$output_dir: file already exists but is not a directory."
      exit 43
    fi
  fi
  mkdir -p "$output_dir"
  o=$output_dir
  
  echo "* Testing `basename $acl`..."
  $nupyf -d $o/dispatch_rules -f $o/forward_rules -i $o/input_rules -o $o/output_rules -n $o/nat_rules -m $o/mangle_rules --ulog --iptables=$iptables --sortid=1 $test_dir/desc.xml $acl
  failure=1
  expect_dir=`echo $acl | sed 's/^\(.*\)\/acls_\(.*\)\.xml$/\1\/expect_\2/'`
  for f in `ls $output_dir/*`
  do
    tail +2 $f > $f.tail
    mv -f $f.tail $f
    diff -u $expect_dir/`basename $f` $f
    diff_code=$?
    if [ ! $diff_code = 0 ]
    then
      failure=0
      echo "DIFF IN $f"
    fi
  done
  if [ ! $failure = 0 -o $delete_output = 0 ]
  then
    rm -rf "$output_dir"
  fi
  if [ $failure = 0 ]
  then
    # echo $nupyf -d $o/dispatch_rules -f $o/forward_rules -i $o/input_rules -o $o/output_rules -n $o/nat_rules -m $o/mangle_rules --ulog --iptables=$iptables --sortid=1 $test_dir/desc.xml $acl
    echo "[FAIL]"
    if [ ! $continue_mode = 0 ]
    then
      exit 42
    fi
  else
    echo "[ OK ]"
  fi
}

function test_case {
  echo "------------------------------------------------------------------------------"
  echo "ENTERING INTO $1 TESTCASE"
  test_dir=$script_dir/$1
  if [ ! -d "$test_dir" ]
  then
    echo "$test_dir: no such directory."
    return
  fi
  if [ ! -f "$test_dir/desc.xml" ]
  then
    echo "$test_dir/desc.xml: file not found."
    return
  fi
  for acl in `ls $test_dir/acls*.xml`
  do
    test_file $acl
  done
}

delete_output=0
continue_mode=1
while getopts "cdh" flag
do
  #echo "flag: $flag $OPTARG"
  case "$flag" in
    'h') print_usage; exit 0;;
    'd') delete_output=1;;
    'c') continue_mode=0;;
    '?') echo "Bad option: $flag"; print_usage; exit 1;;
  esac
done
shift $(($OPTIND-1))

if [ $# = 0 ]
then
  testcases=`cd $script_dir && ls -d */`
else
  testcases=$*
fi

for i in $testcases
do
  test_case $i
done
echo "------------------------------------------------------------------------------"
